@@ -1,9 +1,29 @@
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+/* wvWare
+ * Copyright (C) Caolan McNamara, Dom Lachowicz, and others
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
 #include "wv.h"
 
 /*
@@ -350,11 +370,15 @@ wvAssembleSimplePAP (wvVersion ver, PAP * apap, U32 fc, PAPX_FKP * fkp, wvParseS
 	means that we now have to apply the list PAPX over what we have
 	and then reapply the block PAPX (we had to apply the block's PAPX
 	in order to find out if we are in a list !!!)*/
+
+	if (!ps->lfo)
+	  return ret;
 	
 	wvTrace(("list: ilvl %d, ilfo %d\n",apap->ilvl,apap->ilfo));	/* ilvl is the list level */
 
 	/* first, get the LFO, and then find the lfovl for this paragraph */
-	myLFO = &ps->lfo[apap->ilfo - 1];
+	if (ps->lfo) 
+	  myLFO = &ps->lfo[apap->ilfo - 1];
 
 	while(i < (S32)apap->ilfo - 1 && i < (S32)ps->nolfo)
 	{
@@ -363,7 +387,10 @@ wvAssembleSimplePAP (wvVersion ver, PAP * apap, U32 fc, PAPX_FKP * fkp, wvParseS
 	}
 
 	/* 	remember how many overrides are there for this record */
-	k = ps->lfo[i].clfolvl;
+        if (ps->lfo)
+	  k = ps->lfo[i].clfolvl;
+	else
+	  k = 0;
 
 	/* 	if there are any overrides, then see if one of them applies to this level */
 	if(k && ps->lfolvl)
@@ -454,7 +481,7 @@ wvAssembleSimplePAP (wvVersion ver, PAP * apap, U32 fc, PAPX_FKP * fkp, wvParseS
 	{
 		prevLVL = myLVL;
 		prevLVLF = myLVLF;
-		myListId = myLFO->lsid;
+		myListId = myLFO ? myLFO->lsid : 0;
 		wvTrace(("list: using the LVL from LST\n"));
 		i = 0;
 		
