@@ -1,11 +1,31 @@
-#include <stdio.h>
-#include <stdlib.h>
+/* wvWare
+ * Copyright (C) Caolan McNamara, Dom Lachowicz, and others
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include "wv.h"
-#include "iconv.h"
+#include <glib.h>
 
 /*modify this to handle cbSTSHI < the current size*/
 void
@@ -111,6 +131,31 @@ wvInitSTD (STD * item)
     item->grupe = NULL;
 }
 
+static const char * wvGetUCS2LEName(void)
+{
+  char ** p;
+
+  static char * szUCS2LENames[] = {
+    "UCS-2LE",			/* preferred */
+    "UCS-2-LE",			/* older libiconv */
+    "UNICODELITTLE",	        /* older glibc */
+    "UTF-16LE",			/* superset */
+    "UTF-16-LE",		/* my guess */
+    NULL };
+
+  for (p = szUCS2LENames; *p; ++p)
+    {
+      GIConv g_iconv_handle;
+      if ((g_iconv_handle = g_iconv_open(*p,*p)) != (GIConv)-1)
+	{
+	  g_iconv_close(g_iconv_handle);
+	  return *p;
+	}
+    }
+
+  return NULL;
+}
+
 int
 wvGetSTD (STD * item, U16 baselen, U16 fixedlen, wvStream * fd)
 {
@@ -120,7 +165,7 @@ wvGetSTD (STD * item, U16 baselen, U16 fixedlen, wvStream * fd)
     int ret = 0;
     U16 count = 0;
     U32 allocName = 0;		/* length allocated for xstzName */
-    iconv_t conv = NULL;
+    GIConv conv = NULL;
 	U32 b = 0;
 
     wvInitSTD (item);		/* zero any new fields that might not exist in the file */
@@ -188,14 +233,14 @@ wvGetSTD (STD * item, U16 baselen, U16 fixedlen, wvStream * fd)
 	    }
       }
 
-
     wvTrace (("doing a std, str len is %d\n", len + 1));
     allocName = (len + 1) * sizeof (char);
     item->xstzName = (char *) wvMalloc (allocName);
 	*(item->xstzName) = 0;
 	b = 0;
 
-    conv = iconv_open("utf-8", "UCS-2");
+    conv = g_iconv_open("utf-8", wvGetUCS2LEName ());
+
     for (i = 0; i < len + 1; i++)
       {
 	  if (count < 10)
@@ -207,31 +252,34 @@ wvGetSTD (STD * item, U16 baselen, U16 fixedlen, wvStream * fd)
 	  else
 	    {
 		char buf[16];
+		char buf2[4];
 		char  * tmp;
-		const char * tmp2;
+		char * tmp2;
 		size_t insz, sz;
 		temp16 = read_16ubit (fd);
-		tmp2 = (char *)&temp16;
-		insz = sizeof(temp16);
+		buf2[0] =  temp16 & 0x00ff;
+		buf2[1] = (temp16 & 0xff00) >> 8;
+		tmp2 = buf2;
+		insz = 2 * sizeof(char);
 		tmp = buf;
 		sz =  sizeof(buf);
-		iconv (conv, &tmp2, &insz, &tmp, &sz);
-		while ((b + sizeof(buf) - sz + 1) >= allocName) {
-			allocName *=  2;
-			item->xstzName = (char *) realloc(item->xstzName, allocName);
+		if ((size_t) -1 != g_iconv (conv, &tmp2, &insz, &tmp, &sz)) {
+		  while ((b + sizeof(buf) - sz + 1) >= allocName) {
+		    allocName *=  2;
+		    item->xstzName = (char *) realloc(item->xstzName, allocName);
+		  }
+		  if (sz) {
+		    *tmp = 0;
+		  }
+		  strncat (item->xstzName, buf, sizeof(buf) - sz);
+		  b += (sizeof(buf) - sz);
+		  pos += 2;
 		}
-		if (sz) {
-			*tmp = 0;
-		}
-		strncat (item->xstzName, buf, sizeof(buf) - sz);
-		b += (sizeof(buf) - sz);
-		pos += 2;
-
 	    }
 
 	  wvTrace (("sample letter is %c\n", item->xstzName[i]));
       }
-    iconv_close(conv);
+    g_iconv_close(conv);
     wvTrace (("string ended\n"));
 
 
