@@ -1,8 +1,28 @@
-#include <stdlib.h>
-#include <stdio.h>
+/* wvWare
+ * Copyright (C) Caolan McNamara, Dom Lachowicz, and others
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <stdlib.h>
+#include <stdio.h>
 #include "wv.h"
 #include "bintree.h"
 
@@ -104,7 +124,10 @@ wvGetFullTableInit (wvParseStruct * ps, U32 para_intervals, BTE * btePapx,
 	  i = para_fcLim;
 
 	  /* ignore the row end markers */
-	  if (apap.ptap.itcMac)
+	  /*if (apap.ptap.itcMac)*/
+	  /* we ascertain the number of rows by counting the end of row
+		 markers. NB: a row marker can have a 0 itcMac*/
+	  if (apap.fTtp)
 	    {
 		test = (TAP *) realloc (test, sizeof (TAP) * (j + 1));
 		wvCopyTAP (&(test[j]), &apap.ptap);
@@ -180,7 +203,7 @@ wvSetTableInfo (wvParseStruct * ps, TAP * ptap, int no)
 
     for (i = 0; i < no; i++)
       {
-	  for (j = 0; j < ptap[i].itcMac + 1; j++)
+		  for (j = 0; j < ptap[i].itcMac + 1; j++)
 	    {
 		wvTrace (("%d\n", ptap[i].rgdxaCenter[j]));
 		InsertNode (&tree, (void *) &(ptap[i].rgdxaCenter[j]));
@@ -811,7 +834,10 @@ wvGetComplexFullTableInit (wvParseStruct * ps, U32 para_intervals,
 	  i = para_fcLim;
 
 	  /* ignore the row end markers */
-	  if ((apap.ptap.itcMac) && (apap.fTtp))
+	  /*  if ((apap.ptap.itcMac) (apap.fTtp))*/
+	  /* we ascertain the number of rows by counting the end of row
+		 markers. NB: a row marker can have a 0 itcMac*/
+	  if (apap.fTtp)
 	    {
 		test = (TAP *) realloc (test, sizeof (TAP) * (j + 1));
 		wvCopyTAP (&(test[j]), &apap.ptap);
