@@ -1,24 +1,35 @@
+/* wvWare
+ * Copyright (C) Caolan McNamara, Dom Lachowicz, and others
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include "iconv.h"
 #include "wv.h"
-
-extern TokenTable s_Tokens[];
+#include <glib.h>
 
 int (*wvConvertUnicodeToEntity) (U16 char16) = NULL;
 
-/* i hate iconv - compilers treat its prototype differently */
-#if !defined(WIN32) || !defined(_WIN32)
-#define wv_iconv(a,b,c,d,e) iconv(a, (ICONV_CONST char**)b,c,(char**)d,e)
-#else
-#define wv_iconv(a,b,c,d,e) iconv(a,b,c,(char**)d,e)
-#endif
-
 U16
 wvnLocaleToLIDConverter (U8 nLocale)
 {
@@ -150,15 +161,15 @@ wvOutputHtmlChar (U16 eachchar, U8 chartype, char *outputtype, U16 lid)
        static char* cpname = NULL;                             \
        if (!cpname)    \
                {       \
-                       iconv_t cd = iconv_open(name,name);     \
-                       if (cd==(iconv_t)-1)    \
+                       GIConv cd = g_iconv_open(name,name);     \
+                       if (cd==(GIConv)-1)    \
                                {       \
                                        cpname = fallbackname;  \
                                }       \
                        else    \
                                {       \
                                        cpname = name;  \
-                                       iconv_close(cd);        \
+                                       g_iconv_close(cd);        \
                                }       \
                };      \
        return cpname;  \
@@ -175,7 +186,7 @@ static const wvLanguageId mLanguageIds[] =
 		{ "-none-", 0x0400 }, /* none */
 		{ "af-ZA", 0x0436 }, /* Afrikaans */
 		{ "am", 0x045e }, /* Amharic */
-		{ "al-AL", 0x041c }, /* Albanian */
+		{ "sq-AL", 0x041c }, /* Albanian */
 		{ "ar-SA", 0x0401 }, /* Arabic (Saudi) */
 		{ "ar-IQ", 0x0801 }, /* Arabic (Iraq) */
 		{ "ar-EG", 0x0c01 }, /* Arabic (Egypt) */		
@@ -577,7 +588,7 @@ wvLIDToCodePageConverter (U16 lid)
 static U32
 swap_iconv (U16 lid)
 {
-    iconv_t handle = NULL;
+    GIConv handle = NULL;
     char f_code[33];		/* From CCSID                           */
     char t_code[33];		/* To CCSID                             */
     const char *codepage = NULL;
@@ -586,7 +597,7 @@ swap_iconv (U16 lid)
     U8 buffer[2];
     U8 buffer2[2];
 
-    U8 *ibuf, *obuf;
+    gchar *ibuf, *obuf;
 
     /* do a bit of caching */
     static U16 lastlid = -1;
@@ -608,8 +619,8 @@ swap_iconv (U16 lid)
     strcpy (f_code, codepage);
     strcpy (t_code, "UCS-2");
 
-    handle = iconv_open (t_code, f_code);
-    if (handle == (iconv_t)-1)
+    handle = g_iconv_open (t_code, f_code);
+    if (handle == (GIConv)-1)
             return 0;
 
     buffer[0] = 0x20 & 0xff;
@@ -617,9 +628,9 @@ swap_iconv (U16 lid)
 
     ibuflen = obuflen = 2;
 
-    wv_iconv (handle, &ibuf, &ibuflen, &obuf, &obuflen);
+    g_iconv (handle, &ibuf, &ibuflen, &obuf, &obuflen);
 
-    iconv_close (handle);
+    g_iconv_close (handle);
 
     ret = *(U16 *) buffer2 != 0x20;
     return ret;
@@ -631,13 +642,13 @@ wvHandleCodePage (U16 eachchar, U16 lid)
     char f_code[33];		/* From CCSID                           */
     char t_code[33];		/* To CCSID                             */
     const char *codepage;
-    iconv_t iconv_handle;	/* Conversion Descriptor returned       */
-    /* from iconv_open() function           */
+    GIConv g_iconv_handle;	/* Conversion Descriptor returned       */
+    /* from g_iconv_open() function           */
     size_t ibuflen;		/* Length of input buffer               */
     size_t obuflen;		/* Length of output buffer              */
 
-    U8 *ibuf;
-    U8 *obuf;			/* Buffer for converted characters      */
+    gchar *ibuf;
+    gchar *obuf;			/* Buffer for converted characters      */
     U8 *p;
     U8 buffer[2];
     U8 buffer2[2];
@@ -669,11 +680,11 @@ wvHandleCodePage (U16 eachchar, U16 lid)
     strcpy (f_code, codepage);
     strcpy (t_code, "UCS-2");
 
-    iconv_handle = iconv_open (t_code, f_code);
-    if (iconv_handle == (iconv_t) - 1)
+    g_iconv_handle = g_iconv_open (t_code, f_code);
+    if (g_iconv_handle == (GIConv) - 1)
       {
 	  wvError (
-		   ("iconv_open fail: %d, cannot convert %s to unicode\n",
+		   ("g_iconv_open fail: %d, cannot convert %s to unicode\n",
 		    errno, codepage));
 	  return ('?');
       }
@@ -681,7 +692,7 @@ wvHandleCodePage (U16 eachchar, U16 lid)
     ibuflen = obuflen = 2;
     p = obuf;
 
-    wv_iconv (iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
+    g_iconv (g_iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
 
     /* We might have double byte char here. */
 
@@ -695,7 +706,7 @@ wvHandleCodePage (U16 eachchar, U16 lid)
 	  rtn = *(U16 *) buffer2;
       }
 
-    iconv_close (iconv_handle);
+    g_iconv_close (g_iconv_handle);
 
     return (rtn);
 }
@@ -704,9 +715,9 @@ void
 wvOutputFromUnicode (U16 eachchar, char *outputtype)
 {
     static char cached_outputtype[33];	/* Last outputtype                  */
-    static iconv_t iconv_handle = (iconv_t)-1;	/* Cached iconv descriptor          */
+    static GIConv g_iconv_handle = (GIConv)-1;	/* Cached iconv descriptor          */
     static int need_swapping;
-    U8 *ibuf, *obuf;
+    gchar *ibuf, *obuf;
     size_t ibuflen, obuflen, len, count, i;
     U8 buffer[2], buffer2[5];
 
@@ -714,16 +725,16 @@ wvOutputFromUnicode (U16 eachchar, char *outputtype)
 	&& wvConvertUnicodeToEntity (eachchar))
 	return;
 
-    if ((iconv_handle == (iconv_t)-1) || strcmp (cached_outputtype, outputtype) != 0)
+    if ((g_iconv_handle == (GIConv)-1) || strcmp (cached_outputtype, outputtype) != 0)
       {
-	  if ((iconv_handle != (iconv_t)-1))
-	      iconv_close (iconv_handle);
+	  if ((g_iconv_handle != (GIConv)-1))
+	      g_iconv_close (g_iconv_handle);
 
-	  iconv_handle = iconv_open (outputtype, "UCS-2");
-	  if (iconv_handle == (iconv_t) - 1)
+	  g_iconv_handle = g_iconv_open (outputtype, "UCS-2");
+	  if (g_iconv_handle == (GIConv) - 1)
 	    {
 		wvError (
-			 ("iconv_open fail: %d, cannot convert %s to %s\n",
+			 ("g_iconv_open fail: %d, cannot convert %s to %s\n",
 			  errno, "UCS-2", outputtype));
 		printf ("?");
 		return;
@@ -742,7 +753,7 @@ wvOutputFromUnicode (U16 eachchar, char *outputtype)
 	  ibuflen = 2;
 	  obuflen = 5;
 
-	  count = wv_iconv (iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
+	  count = g_iconv (g_iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
 	  if (count >= 0)
 	      need_swapping = buffer2[0] != 0x20;
       }
@@ -764,7 +775,7 @@ wvOutputFromUnicode (U16 eachchar, char *outputtype)
     ibuflen = 2;
     len = obuflen = 5;
 
-    count = wv_iconv (iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
+    count = g_iconv (g_iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
     if (count == (size_t) - 1)
       {
 	  wvError (("iconv failed errno: %d, char: 0x%X, %s -> %s\n",
@@ -2160,7 +2171,7 @@ char *
 wvConvertStylename(char *stylename, char *outputtype)
 {
     static char cached_outputtype[36];
-    static iconv_t iconv_handle = (iconv_t)-1;
+    static GIConv g_iconv_handle = (GIConv)-1;
     /**FIXME: 100 is just the size of stylename[] from wv.h**/
     static char buffer[100];
     char *ibuf, *obuf;
@@ -2169,23 +2180,23 @@ wvConvertStylename(char *stylename, char *outputtype)
     /* Destroy */
     if(!outputtype) 
     {
-	if ((iconv_handle != (iconv_t)-1))
-	    iconv_close(iconv_handle);
+	if ((g_iconv_handle != (GIConv)-1))
+	    g_iconv_close(g_iconv_handle);
 	return NULL;
     }
 
     /* Initialize */
-    if(!iconv_handle || strcmp(cached_outputtype, outputtype))
+    if(!g_iconv_handle || strcmp(cached_outputtype, outputtype))
     {
-	if ((iconv_handle != (iconv_t)-1))
-	    iconv_close(iconv_handle);
+	if ((g_iconv_handle != (GIConv)-1))
+	    g_iconv_close(g_iconv_handle);
 
 	/**FIXME: dont know if ISO-8859-1 is really the correct
 	 **charset for style names with eg umlauts.             **/
-	iconv_handle = iconv_open(outputtype, "ISO-8859-1");
-	if(iconv_handle == (iconv_t)-1)
+	g_iconv_handle = g_iconv_open(outputtype, "ISO-8859-1");
+	if(g_iconv_handle == (GIConv)-1)
 	{
-	    wvError(("iconv_open fail: %d, cannot convert %s to %s\n",
+	    wvError(("g_iconv_open fail: %d, cannot convert %s to %s\n",
 		     errno, "ISO-8859-1", outputtype));
 	    return stylename;
 	}
@@ -2198,7 +2209,7 @@ wvConvertStylename(char *stylename, char *outputtype)
     ibuflen = strlen(stylename);
     obuf    = buffer;
     obuflen = sizeof(buffer) - 1;
-    len     = wv_iconv (iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
+    len     = g_iconv (g_iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
     *obuf   = 0;
     if(len == -1) 
     {
