@@ -1350,7 +1350,7 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 			achp->fcPic_fcObj_lTagObj, achp->fObj, achp->fOle2));
 
 	      if (achp->fOle2)
-		  exit (139);
+		return (0);
 	      if(!no_graphics) 
 	      {
 	      wvStream_goto (ps->data, achp->fcPic_fcObj_lTagObj);
@@ -1607,7 +1607,7 @@ myCharProc (wvParseStruct * ps, U16 eachchar, U8 chartype, U16 lid)
 int
 wvOpenConfig (state_data *myhandle,char *config)
 {
-    char buf[BUFSIZ];
+    static char buf[BUFSIZ] = "";
     FILE *tmp;
     int i = 0;
     if (config == NULL)
